#!/usr/bin/perl
##
## Package:   GeekTools Whois Proxy 3.1.7
## File:      proxy.pl (inetd)
## Author:    Robb Ballard <robb@centergate.com>
## Comments:  Part of the GeekTools Whois Proxy package.  See
##            the file README for more information.
##
## Copyright: CenterGate Research Group, LLC - 2001 All Rights Reserved
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## If you use this program to provide publicly available output, please
## give us some sort of credit in the results.  If you don't like the way
## the script does something, rewrite it and send us a diff instead of
## complaining.


use IO::Socket;
use Net::DNS;
use AnyDBM_File;

$SIG{ALRM}  = sub { die "timeout" };

## 
## Configuration
##

## Notification mail address (change this to your own or an alias)
my $NotifyMail = "you\@yourdomain.com";

## Location of TLD/Whois server list.
my $RSList = "/usr/local/geektools/whoislist";

## Access Control List
$AccessList  = "/usr/local/geektools/accesslist";

## Maximum number of hits per day from a single IP address.
my $HitLimit = 50;

## Global hit counter (per site)
my $SiteHits;

## Location of the access (hit) database.
$AccessDB = "/var/tmp/gtwhois-hits";

## Default whois server (don't undef this)
$DefaultNic = "whois.networksolutions.com";

## Grab the incoming query and strip off any control characters at the end.

my $QueryIn = <STDIN>;
chomp $QueryIn;

## Track hits from addresses to prevent abuse (see above).  In this case
## tcpd provides the client IP address as an argument.  Comment this out
## if you're not going to keep track of who uses your proxy.

my $RemoteHost = @ARGV[0];
@AA = split /\./, $RemoteHost;
$IPAddress = pack('C4',@AA);
my $RemoteHostName = gethostbyaddr($IPAddress,2);
if ($RemoteHostName eq "") { $RemoteHostName = $RemoteHost; }
&CheckAccess();

## End of address collection block.

if ($QueryIn eq "") {
   &ShowResults();
   exit 0;
}

my @QueryArray    = split /\./, $QueryIn;
my $QueryElements = @QueryArray;

my $OrigQuery     = $QueryIn;

$QueryIn       =~ tr/A-Z/a-z/;

my @SuffixArray   = split /\ /,$QueryIn;
my $SuffixElements= @SuffixArray - 1;
$SuffixArray[$SuffixElements] =~ tr/a-z.,0-9\-\+//cd;
$QueryIn       =~ tr/a-z.,0-9\-\@\!\ \+\:\///cd;

## Extra spaces on the end of the query?  Kill them.
$QueryIn =~ s/^\s+//;
$QueryIn =~ s/\s+$//;

##
## Data dissection
##

## Suffix queries

my %SuffixMap = (
	"ra"		=> "whois.ra.net",
	"arin"		=> "whois.arin.net",
	"apnic"    	=> "whois.apnic.net",
	"aunic"    	=> "whois.aunic.net",
	"ripe"     	=> "whois.ripe.net",
	"register" 	=> "whois.register.com",
	"nsi" 		=> "whois.networksolutions.com",
	"crsnic"   	=> "whois.crsnic.net",
	"abuse"    	=> "whois.abuse.net",
	"asn"      	=> "whois.arin.net",
	"net"      	=> "whois.arin.net",
	"core"     	=> "whois.corenic.net",
	"ap"       	=> "whois.apnic.net",
	"6bone"       	=> "whois.6bone.net",
	"netblk"   	=> "whois.arin.net"
	);

if (exists($SuffixMap{$SuffixArray[$SuffixElements]})) {
   my $AddUp = 0;
   my $SuffixQuery = "";
   while ($AddUp < $SuffixElements) {
         $SuffixQuery = $SuffixQuery . " " . $SuffixArray[$AddUp];
         ++$AddUp;
   }
   $SuffixQuery =~ s/ //g;
   $NicServ = $SuffixMap{$SuffixArray[$SuffixElements]};
   if ($NicServ ne "whois.abuse.net") {
       &Whois($SuffixMap{$SuffixArray[$SuffixElements]}, "$SuffixQuery");
   } else {
       $WhoisResults = "Information from abuse.net:\n";
       &Whois("whois.abuse.net", "$SuffixQuery");
       $WhoisResults = $WhoisResults . "\nWHOIS information:\n";
       $NoClear = "Y";
       &DomainDig($SuffixQuery);
       
   } 

   &ShowResults();
}

## Single element queries
if ($QueryElements eq 1) {
   ## Single numerics are interpreted as AS numbers or obfuscated addresses.
   if ($QueryIn !~ /[^0-9]/) {
      ## Obfuscated?
      if ($QueryIn > 20000) {
         &Unobfuscate($QueryIn);
      }
      &CheckAS($QueryIn);
      &ShowResults();
   }
   
   ## Hyphenated data.
   my ($check1,$check2,$check3) = split /\-/, $QueryIn;
   ## After the hyphen.
   $_ = $check2;
   /^arin/i	&&	( $NicServ = "whois.arin.net" );
   /^norid/i	&&	( $NicServ = "whois.norid.no" );
   /^ripe/i	&&	( $NicServ = "whois.ripe.net" );
   /^ap/i	&&	( $NicServ = "whois.apnic.net" );
   /^hst/i	&&	( $NicServ = "whois.networksolutions.com" );
   /^dom/i	&&	( $NicServ = "whois.networksolutions.com" );
   ## multi-segmented (usually role) handles -- there should be a 
   ## better way to do this.
   $_ = $check3;
   /^arin/i     &&      ( $NicServ = "whois.arin.net" );
   /^ripe/i     &&      ( $NicServ = "whois.ripe.net" );
   /^ap/i       &&      ( $NicServ = "whois.apnic.net" );
   /^hst/i      &&      ( $NicServ = "whois.networksolutions.com" );
   /^dom/i      &&      ( $NicServ = "whois.networksolutions.com" );
   ## Before the hyphen. 
   $_ = $check1;
   /^asn/i	&&	( $NicServ = "whois.arin.net" );
   /^net/i	&&	( $NicServ = "whois.arin.net" );
   /^netblk/i	&&	( $NicServ = "whois.arin.net" );
   /^coco/i	&&	( $NicServ = "whois.corenic.net" );
   /^coho/i	&&	( $NicServ = "whois.corenic.net" );
   ##
   ##  JP handles - whois.nic.ad.jp has handles ending in JP, no hyphen.
   ##
   my $look = length $QueryIn;
   $look = $look-2;
   my $last = substr $QueryIn, $look, 2;
   if ($last eq "jp") { $NicServ = "whois.nic.ad.jp"; }

   ## IPv6
   if ($QueryIn =~ /^[0-9a-f][0-9a-f][0-9a-f][0-9a-f]:/) { 
      &IPV6Lookup("$QueryIn"); 
   }

   ## NORID wants handle queries in upper case only
   if ($NicServ eq "whois.norid.no") {
      $QueryIn = $OrigQuery;
   }

   # Do the query:
   &Whois("$NicServ","$QueryIn");
   &ShowResults();
}

## IP and domain checking.
if ($QueryIn !~ /[^0-9\.]/) {
   ($a, $b, $c, $d, $e) = $QueryIn =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)(.*)/;
    if ($a > 255 || $b > 255 || $c > 255 || $d > 255 || $e ne '') {
       ShowError("Unable to decipher your entry, bailing out.");
       exit 1;
    }
    &IPCheck("$QueryIn");
    } else {
    &DomainDig("$QueryIn");
}


##
## Subroutines
##

# This is an ugly hack to get .org whois lookups working again.  Apparently
# whois.publicinterestregistry.net is going to be a "thick" registry like
# NetSol but for now they only have munged pointers to where the real
# whois data is.  This nasty bit of code tries to pick out where the real
# whois server is and get information from it.

sub WhichOrgWhois {
   my $Remote;
   my $MyQuery = $_[0];
   my $NicServ;
   my $Port   = getservbyname 'whois', 'tcp';

   # 20 second failsafe if there is no connection and the socket code
   # doesn't handle it.

   eval {
        alarm(20);
        $Remote = IO::Socket::INET->new(
                Proto=>'tcp',
                PeerAddr=>'whois.publicinterestregistry.net',
                PeerPort=>$Port,
                Timeout=>'15'
                );
        alarm(0);
        };

   if ($@ =~ /timeout/) {
      &ShowError("Unable to connect, please try again later.");
   }

   if (!$Remote) {
      &ShowError("Unable to contact whois.publicinterestregistry.net.\r\n");
   }
   print $Remote "$MyQuery\r\n";
   my @Result = <$Remote>;
   close $Remote;

   foreach my $ResultLine (@Result) {
       $ResultLine =~ s/\015\012/\012/g;
       $ResultLine =~ s/\n$//;
       my ($A,$B,$C) = split /\:/, $ResultLine;
       if ($B eq "Whois Server") { $NicServ = $C; }
   }
   $NicServ =~ tr/A-Z/a-z/;
   $NicServ =~ tr/a-z.\-,0-9//cd;

   ## Default to $DefaultNic if crsnic has nothing.
   if ($NicServ eq "") { $NicServ = $DefaultNic; }

   return($NicServ);
}


sub WhichWhois {
   my $Remote;
   my $MyQuery = $_[0];
   my $NicServ;
   my $Port   = getservbyname 'whois', 'tcp';

   # 20 second failsafe if there is no connection and the socket code
   # doesn't handle it.
  
   eval {
        alarm(20);
        $Remote = IO::Socket::INET->new(
                Proto=>'tcp',
                PeerAddr=>'whois.crsnic.net',
                PeerPort=>$Port,
                Timeout=>'15'
                );
        alarm(0);
        };

   if ($@ =~ /timeout/) {
      &ShowError("Unable to connect, please try again later.");
   }
         
   if (!$Remote) {
      &ShowError("Unable to connect to whois.crsnic.net.");
   }
   print $Remote "=$MyQuery\r\n";
   my @Result = <$Remote>;
   close $Remote;
 
   foreach my $ResultLine (@Result) {
       my ($A,$B) = split /\:/, $ResultLine;
       $A =~ s/ //g;
       $B =~ s/ //g;
       if ($A eq "WhoisServer") { $NicServ = $B; }
   }
   $NicServ =~ tr/A-Z/a-z/;
   $NicServ =~ tr/a-z.\-,0-9//cd;

   ## Default to $DefaultNic if crsnic has nothing.
   if ($NicServ eq "") { $NicServ = $DefaultNic; }

   return($NicServ);
}

sub IPCheck {
   ## Start with ARIN.  $Port can be defined now because we know about
   ## ARIN's server.  $Port may be redefined later if we find an rwhois
   ## server to talk to.

   $IPQuery = $_[0];

   $NicServ = "whois.arin.net";
   my $Referer;
   my $Remote;
   my $Tries = 1;
   my $MaxRecurse = 5;
   my $Port = getservbyname 'whois', 'tcp';
   REPEAT: while ($IPQuery) {
           eval {
           alarm(30);
           $Remote = IO::Socket::INET->new(
                Proto           => 'tcp',
                PeerAddr        => $NicServ,
                PeerPort        => $Port,
                Timeout         => '20'
                );
           alarm(0);
           };

           if ($@ =~ /timeout/) {
              &ShowError("Unable to connect, please try again later.");
           }

           if ($NicServ eq "whois.nic.ad.jp") { $IPQuery = $IPQuery . "/e"; }

           if (!$Remote) {
              &ShowError("Unable to connect to $NicServ.");
           }
 
           if ($Tries > $MaxRecurse) {
              &ShowError("Possible query loop, max tries exceeded.  Exiting.");
           }

           ++$Tries;

           #if ($NicServ eq "whois.arin.net") {
              print $Remote "$IPQuery\r\n";
           #}
           @Result = <$Remote>;
           close $Remote;

           ## Look through the results for clues about who to contact
           ## next (if needed).  We're assuming some things here but
           ## they might be right.  If not.... ;-)
           $NewNic = $NicServ;

           foreach (@Result) {
             /.*LACNIC.*/           && ($NewNic = "whois.lacnic.net");
             /.*APNIC.*/            && ($NewNic = "whois.apnic.net");
             /.*APNIC-.*/            && ($NewNic = "whois.apnic.net");
             /.*AUNIC-AU.*/         && ($NewNic = "whois.aunic.net");
             /.*NETBLK-RIPE.*/      && ($NewNic = "whois.ripe.net");
             /.*NETBLK-.*-RIPE.*/   && ($NewNic = "whois.ripe.net");
             /.*NET-RIPE.*/         && ($NewNic = "whois.ripe.net");
             /.*-RIPE.*/            && ($NewNic = "whois.ripe.net");
             /.*RIPE-.*/            && ($NewNic = "whois.ripe.net");
             /.*NETBLK-BRAZIL.*/    && ($NewNic = "whois.nic.br");
             /.*whois\.nic\.ad\.jp.*/i && ($NewNic = "whois.nic.ad.jp");
             /.*whois\.telstra.*/i  && ($NewNic = "whois.telstra.net");
             /.*rwhois\.exodus.*/i  && ($NewNic = "rwhois.exodus.net");
             /.*rwhois\.verio.*/i  && ($NewNic = "rwhois.verio.net");
             /.*rwhois\.dnai.*/i    && ($NewNic = "rwhois.dnai.com");
             /.*rwhois\.digex.*/i   && ($NewNic = "rwhois.digex.net");
             /.*rwhois\.internex.*/i   && ($NewNic = "rwhois.internex.net");
             /.*rwhois\.concentric.*/i && ($NewNic = "rwhois.concentric.net");
             # rwhois.eni.net is currently broken
             # /.*rwhois\.eni.*/i     && ($NewNic = "rwhois.eni.net");
             /.*rwhois\.oar.*/i     && ($NewNic = "rwhois.oar.net");
             /.*rwhois\.elan.*/i     && ($NewNic = "rwhois.elan.net");
             /.*rwhois\.cais.*/i     && ($NewNic = "rwhois.cais.net");
             /.*rwhois\.cogentco.*/i && ($NewNic = "rwhois.cogentco.com");
             /.*rwhois\.beanfield.*/i && ($NewNic = "rwhois.beanfield.net");
             /.*JPNIC*.*/           && ($NewNic = "whois.nic.ad.jp");
             /.*JNIC*.*/            && ($NewNic = "whois.nic.ad.jp");
             /.*whois.nic.or.kr.*/i && ($NewNic = "whois.nic.or.kr");

             if ($NewNic ne $NicServ) {
                $NicServ = $NewNic;
                ## Screwball port requirements and rwhois servers are
                ## are taken care of here.
		
		if ($NicServ eq "rwhois.exodus.net") { $Port = "4321"; }
                if ($NicServ eq "rwhois.verio.net") { $Port = "4321"; }
                if ($NicServ eq "rwhois.dnai.com")   { $Port = "4321"; }
                if ($NicServ eq "rwhois.digex.net")  { $Port = "4321"; }
                if ($NicServ eq "rwhois.internex.net")    { $Port = "4321"; }
                if ($NicServ eq "rwhois.concentric.net")  { $Port = "4321"; }
                if ($NicServ eq "rwhois.eni.net")  { $Port = "4321"; }
                if ($NicServ eq "rwhois.oar.net")  { $Port = "4321"; }
                if ($NicServ eq "rwhois.elan.net")  { $Port = "4321"; }
                if ($NicServ eq "rwhois.cais.net")  { $Port = "4321"; }
                if ($NicServ eq "rwhois.beanfield.net")  { $Port = "4321"; }
                if ($NicServ eq "rwhois.cogentco.com")  { $Port = "4321"; }

                if ($Port eq "4321" && $Referer eq "") {
                   ## We have an rwhois server -- save the referring
                   ## data for later display as well.
                   $Referer = "Referring data:\n\n";
                   foreach $_ (@Result) {
                              $Referer = $Referer . $_;
                   }
                   $Referer = $Referer . "\nRwhois server data:\n\n";
                }

                next REPEAT;
             }
           }
      last;
      }
      $WhoisResults = $Referer;
      foreach $_ (@Result) {
              &ZapTrash($_);
              $WhoisResults = $WhoisResults . $_;
      }
      &ShowResults();
}


sub CheckAS {
    ##
    ## AS Lookup Table
    ##
    my $AS = $_[0];
    my $NicServ;
    my $ASQuery;
    if ($AS >= 1 && $AS <= 1876) { $NicServ = "whois.arin.net";  }
    if ($AS >= 1877  && $AS <= 1901) { $NicServ = "whois.ripe.net";  }
    if ($AS >= 1902 && $AS <= 2042) { $NicServ = "whois.arin.net";  }
    if ($AS == 2043) { $NicServ = "whois.ripe.net";  }
    if ($AS >= 2044 && $AS <= 2046) { $NicServ = "whois.arin.net";  }
    if ($AS == 2047) { $NicServ = "whois.ripe.net";  }
    if ($AS >= 2048 && $AS <= 2106) { $NicServ = "whois.arin.net";  }
    if ($AS >= 2107 && $AS <= 2136) { $NicServ = "whois.ripe.net";  }
    if ($AS >= 2137 && $AS <= 2584) { $NicServ = "whois.arin.net";  }
    if ($AS >= 2585 && $AS <= 2614) { $NicServ = "whois.ripe.net";  }
    if ($AS >= 2615 && $AS <= 2772) { $NicServ = "whois.arin.net";  }
    if ($AS >= 2773 && $AS <= 2822) { $NicServ = "whois.ripe.net";  }
    if ($AS >= 2823 && $AS <= 2829) { $NicServ = "whois.arin.net";  }
    if ($AS >= 2830 && $AS <= 2879) { $NicServ = "whois.ripe.net";  }
    if ($AS >= 2880 && $AS <= 3153) { $NicServ = "whois.arin.net";  }
    if ($AS >= 3154 && $AS <= 3353) { $NicServ = "whois.ripe.net";  }
    if ($AS >= 3354 && $AS <= 4607) { $NicServ = "whois.arin.net";  }
    if ($AS >= 4608 && $AS <= 4864) { $NicServ = "whois.apnic.net";  }
    if ($AS >= 4865 && $AS <= 5376) { $NicServ = "whois.arin.net";  }
    if ($AS >= 5377 && $AS <= 5631) { $NicServ = "whois.ripe.net";  }
    if ($AS >= 5632 && $AS <= 6655) { $NicServ = "whois.arin.net";  }
    if ($AS >= 6656 && $AS <= 6911) { $NicServ = "whois.ripe.net";  }
    if ($AS >= 6912 && $AS <= 7466) { $NicServ = "whois.arin.net";  }
    if ($AS >= 7467 && $AS <= 7722) { $NicServ = "whois.apnic.net";  }
    if ($AS >= 7723 && $AS <= 8191) { $NicServ = "whois.arin.net";  }
    if ($AS >= 8192 && $AS <= 9215) { $NicServ = "whois.ripe.net";  }
    if ($AS >= 9216 && $AS <= 10239) { $NicServ = "whois.apnic.net";  }
    if ($AS >= 10240 && $AS <= 11263) { $NicServ = "whois.arin.net";  }
    if ($AS >= 11264 && $AS <= 12287) { $NicServ = "whois.arin.net";  }
    if ($AS >= 12288 && $AS <= 13311) { $NicServ = "whois.ripe.net";  }
    if ($AS >= 13312 && $AS <= 14335) { $NicServ = "whois.arin.net";  }


    ## Structure the query the way the server wants it.
    $_ = $NicServ;
    /^whois.ripe.net/i		&& ( $ASQuery = "AS$QueryIn" );
    /^whois.apnic.net/i		&& ( $ASQuery = "AS$QueryIn" );
    /^whois.arin.net/i		&& ( $ASQuery = "AS $QueryIn" );

    &Whois("$NicServ","$ASQuery");
    &ShowResults();
    exit 0;
}

sub Unobfuscate {
   my $Number = $_[0];
   my $Chunk = sprintf "%lx", $Number;
   ($A,$B,$C,$D) = unpack('C4',pack('N',$Number));
   my $IP = "$A.$B.$C.$D";

   $WhoisResults = "\n\n \
      The unobfuscated address $Number translates to $IP. \
      Attempting to get some information about the IP block assignment. \
      Results, if any, are shown below:\n\n\n";

   &IPCheck($IP);

}

sub CheckAccess {
  ## This supercedes the old checking codes and combines two flat text files
  ## into a single db-type file.  Ok, so it's still a hack.  If you can
  ## do better, don't bitch -- fix it and send me a diff.

  ## Track their usage for today.
  my $Key;
  my $Value;
  my $UData;
  my $UserOK = "N";
  dbmopen(%ACCESSDB, $AccessDB, 0644) ||
         warn "Unable to open access database: $!\n";
  my $DayHits = $ACCESSDB{$RemoteHost};
   if ($DayHits eq "")
      {
        $DayHits = 1;
      } else {
        $DayHits++;
      }
   $ACCESSDB{"$RemoteHost"} = $DayHits;
   dbmclose(%ACCESSDB);
   if ($DayHits <= $HitLimit) { $UserOK = "Y"; }

   ## See if they're in the "list" ... direct match first.
   dbmopen(%ACL, $AccessList, 0644) ||
          warn "Unable to open the ACL database: $!\n";
   $UData = $ACL{"$RemoteHost"};
   ## Now pattern matching.
   while (($Key, $Value) = each(%ACL)) {
         if ($RemoteHost =~ /^$Key.*/) {
            $UData = $Value;
         }
   }

   dbmclose(%ACL); 

   if ($DayHits <= $UData) {
      $UserOK = "Y";   
   } 

   ## New Unlimited and Lockout codes:
   if ($UData eq "L") {
      $UserOK = "N";
   }
   if ($UData eq "U") {
      $UserOK = "Y";
   }

   $SiteHits = $DayHits;
   if ($UserOK eq "N") {
      &AccessDenied();
   }
}

sub ShowResults {

   print "Query:     $QueryIn\n";
   print "Registry:  $NicServ\n";

   print "Results:\n";
   if ($WhoisResults eq "") {
      $WhoisResults = "Null query, no data returned.";
   }
   print "$WhoisResults\n";
   print "Results brought to you by the GeekTools WHOIS Proxy v3.1.7\n";
   print "Server results may be copyrighted and are used with permission.\n";
   if ($RemoteHost ne "") {
      print "Your host ($RemoteHost) has visited $SiteHits times today.\n";
   }
   print "\n\n";
   exit 0;
}

sub Whois {
   my $Remote;
   ($NicServ, $MyQuery) = @_;
   if ($NicServ eq "") { $NicServ = $DefaultNic; }
   my $Port = getservbyname 'whois', 'tcp';
   eval {
        alarm(20);
        $Remote = IO::Socket::INET->new(
                Proto           => 'tcp',
                PeerAddr        => $NicServ,
                PeerPort        => $Port,
                Timeout         => '10'
                );
        alarm(0);
   };

   if ($@ =~ /timeout/) {
      &ShowError("Unable to connect, please try again later.");
   }

   if (!$Remote) {
      &ShowError("Unable to connect to the specified registry $NicServ.");
   }
   if ($NicServ eq "whois.networksolutions.com") {
      print $Remote "L $MyQuery\r\n";
   } else {
      print $Remote "$MyQuery\r\n";
   }
   my @QueryResult = <$Remote>;
   close $Remote;
   ##
   ## Build the output
   ##
   if ($NicServ ne "whois.abuse.net") {$WhoisResults = "";}
   foreach (@QueryResult) {
        chomp $_;
        &ZapTrash($_);
        $WhoisResults = $WhoisResults . $_;
   }
}

sub ShowError {
   print "The GeekTools Whois Proxy has encountered an error: \n";
   print "$_[0]\n";
   print "\nExiting.\n\n";
   exit 1;
}


sub ZapTrash {
   ##
   ## Gets rid of the very annoying NetSol banner and other stuff.
   ##
   chomp($_);
   foreach ($_) {
      /^access to/i       && ( $TrashIt = "Y");
      /^the data in/i     && ( $TrashIt = "Y");
      /^the arin/i        && ( $TrashIt = "Y");
      /^to single/i       && ( $TrashIt = "Y");
      /^handle, shown/i       && ( $TrashIt = "Y");
      /^to optimize query/i   && ( $TrashIt = "Y");

      /^registrant:/i     && ( $TrashIt = "N");
      /^registrant: /i     && ( $TrashIt = "N");
      /^organization:/i     && ( $TrashIt = "N");
      /^organization: /i     && ( $TrashIt = "N");
      /^aborting search/i && ( $TrashIt = "N");
      /^no match/i        && ( $TrashIt = "N");

      /^this policy/i     && ( $TrashIt = "N") && ( $_ = "" );
      /^policy /i         && ( $TrashIt = "N") && ( $_ = "" );
      /^by this policy/i  && ( $TrashIt = "N") && ( $_ = "" );
      /^this query/i      && ( $TrashIt = "N") && ( $_ = "" );
      /^register your/i   && ( $TrashIt = "N") && ( $_ = "" );
      /^get your/i        && ( $TrashIt = "N") && ( $_ = "" );
      /^you agree/i        && ( $TrashIt = "N") && ( $_ = "" );
      /^by submitting/i        && ( $TrashIt = "N") && ( $_ = "" );
      /^the domain council /i        && ( $TrashIt = "N") && ( $_ = "" );
      /^whois HELP\@whois.corenic.net/i  && ( $TrashIt = "N") && ( $_ = "" );
	
      if ($TrashIt eq "Y") { 
          $_ = ""; 
          } else {
          $_ = $_ . "\n";
      }
    }
}

sub AccessDenied {
  
   print "Access Limit Exceeded\n\n";
   print "Your host has exceeded the maximum number of visits per day for \n";
   print "the GeekTools WHOIS Proxy service.  We have placed a limit on \n";
   print "access to help prevent the abuse of this service by spammers and \n";
   print "other lowlife types who like to use the various WHOIS databases \n";
   print "to harvest information.  If you wish to be excluded from this \n";
   print "limitation, please send email to $NotifyMail and \n";
   print "include the following items:\n\n";
   print "- Contact Information.\n";
   print "- The IP address(es) for which you are requesting open access.\n";
   print "- The reason you are requesting more access.\n\n";
   print "If you don't include the above information there may be a delay in\n";
   print "getting your access upgraded.  We apologize for the strict \n";
   print "guidelines but due the the increasing amount of junk floading \n";
   print "around on the 'net we want to do our part to help keep it clean.\n";
   print "\n";
   exit 0;
}

sub TrimDomain {
    my @domain = split /\./, $_[0];
    my $elements = @domain;
    $elements--;
    my $pc = 1;                         ## Skip element 0
    my $qstring = "";
    while ($pc le $elements) {
          if ($pc le $elements && $pc gt 1) {
             $qstring = $qstring . "." . $domain[$pc];
          } else {
             $qstring = $qstring . $domain[$pc];
          }
          $pc++;
    }
    return $qstring;
}

sub DomainDig {
   ##
   ## This convoluted mess "recursively" picks through a query and
   ## tries to find at least something to query for.
   ##
   ($MyQuery) = @_;
   QUERY: @DomainArray = split /\./, $MyQuery;
   my $Elements = @DomainArray;
   --$Elements;
   my $TLD = $DomainArray[$Elements];
   open (RSLIST, $RSList) || &ShowError("$!\nUnable to open $RSList");
   while ($TableData = <RSLIST>) {
         chomp $TableData;
         ($TableTLD,$TableNic,$TableWebsite,$Ex1,$Ex2) = split /\|/, $TableData;
         if ($TableTLD eq $TLD) {
            $NicServ = $TableNic;
            $Website = $TableWebsite . $Ex1 . $Ex2;
         }
   }
   close RSLIST;

   ## Is it a web-only whois server?  If so, give the user what we think is
   ## the correct web page address.
   if ($NicServ eq "WEB") {
      $WhoisResults = " \
          No compatible whois server exists for this domain. \
          Please try $Website for more information.\n\n";
      &ShowResults();
   }

   if ($NicServ eq "") {
      $WhoisResults = " \
          Unable to find any TLD information for this domain. \
          Please check the domain and verify that it is part  \
          of a valid top level domain.\n\n ";
      &ShowResults();
   }

   if ($NicServ eq "NONE") {
      $WhoisResults = " \
          We have no server listed for the requested TLD. \
          If you know if a registry file this TLD that supports \
          either WWW or standard WHOIS client queries, please \
          send mail to $NotifyMail and let us know! \n\n ";
      &CheckSOA($MyQuery);
      &ShowResults();
   } 

   ## Nic specific tweaks and customizations.
   $_ = $MyQuery;
   /\.ac\.uk$/    && ( $NicServ = "whois.ja.net" );
   /\.net\.au$/   && ( $NicServ = "whois.connect.com.au" );
   /\.uk\.com$/   && ( $NicServ = "whois.nomination.net" );
   /\.uk\.net$/   && ( $NicServ = "whois.nomination.net" );
   /\.gb\.com$/   && ( $NicServ = "whois.nomination.net" );
   /\.gb\.net$/   && ( $NicServ = "whois.nomination.net" );
   /\.se\.com$/   && ( $NicServ = "whois.centralnic.com" );
   /\.se\.net$/   && ( $NicServ = "whois.centralnic.com" );

   ## If the whois server is whois.crsnic.net, use it to determine
   ## where the real whois server is.
   if ($NicServ eq "whois.crsnic.net") {
      $NicServ = &WhichWhois($MyQuery);
   }
   ## Same for .org. 
   if ($NicServ eq "whois.publicinterestregistry.net") {
      $NicServ = &WhichOrgWhois($MyQuery);
   }

   ## Comment the following line out if you want Japanese rather than
   ## English.
   if ($NicServ eq "whois.nic.ad.jp") { $Conv = "/e"; }

   if ($NicServ eq "") { $NicServ = $DefaultNic; }
   my $Port = getservbyname 'whois', 'tcp';
   my $Remote;
   eval {
   alarm(20);
   $Remote = IO::Socket::INET->new(
           Proto           => 'tcp',
           PeerAddr        => $NicServ,
           PeerPort        => $Port,
           Timeout         => '10'
           );
   alarm(0);
   };

   if ($@ =~ /timeout/) {
      &ShowError("Unable to connect, please try again later.");
   }

   if (!$Remote) {
      &ShowError("Unable to connect to the specified registry $NicServ.");
   }
   if ($NicServ eq "whois.networksolutions.com") {
      print $Remote "L $MyQuery\r\n";
   } else {
      if ($Conv ne "") {
          print $Remote "$MyQuery $Conv\r\n";
      } else {
          print $Remote "$MyQuery\r\n";
      }
   }

   my @QueryResult = <$Remote>;
   close $Remote;
   $GotMatch = 0;
   foreach (@QueryResult) {

           ## 
           ## Responses from different whois servers so we (sort of) know
           ## when to start cutting pieces off the query.
           ##
           ## NSI-style response (look for "No match")
           /^no match for domain/i && ($GotMatch = 1);
           ## NIC.UK style
           /^no match for/i && ($GotMatch = 1);
           ## RIPE style
           /^\% no entries found/i && ($GotMatch = 1);
           ## cdnnet.ca
           /^not found/i && ($GotMatch = 1);
           ## nic.br & related
	   /^\% no match for/i && ($GotMatch = 1);
           ## domain.kz & related
           /^no entries found/i && ($GotMatch = 1);
           ## whois.frd.ac.za
           /^no information available/i && ($GotMatch = 1);
 	   ## nic-se.se style (either one)
	   /^\# key must be/i && ($GotMatch = 1);
	   /^\# no data found/i && ($GotMatch = 1);
           ## whois.nic.ad.jp
           /^no match!!/i && ($GotMatch = 1);
           ##  nic.as
           /^domain not found/i && ($GotMatch = 1);

   }

   if ($GotMatch eq 1) {
       $MyQuery = &TrimDomain($MyQuery);
       @DomainArray = split /\./, $MyQuery;
       my $Elements = @DomainArray;
       if ($Elements > 1) {
          goto QUERY;
       } else {
          $WhoisResults = "Unable to find any information for your query!";
          &ShowResults();
       }
   } else {
     if ($NoClear ne "Y") {$WhoisResults = "";}
     foreach (@QueryResult) {
             chomp $_;
             &ZapTrash($_);
             $WhoisResults = $WhoisResults . $_;
     }
     &ShowResults();
   }
}

sub IPV6Lookup
{
   my @IPV6 = split /:/, $_;
   if ($IPV6[0] =~ /^3ffe$/ ){
       &Whois("whois.6bone.net", "$_");
       &ShowResults();
    } elsif( $IPV6[0] =~ /^5f[0-f][0-f]$/) {
       &Whois("whois.6bone.net", "$_");
       &ShowResults();
    } elsif( $IPV6[0] =~ /^2001$/ && $IPV6[1] =~ /^2[0-f][0-f]/) {
       &Whois("whois.apnic.net", "$_");
       &ShowResults();
    } elsif( $IPV6[0] =~ /^2001$/ && $IPV6[1] =~ /^4[0-f][0-f]/) {
       &Whois("whois.arin.net", "$_");
       &ShowResults();
    } elsif( $IPV6[0] =~ /^2001$/ && $IPV6[1] =~ /^6[0-f][0-f]/) {
       &Whois("whois.ripe.net", "$_");
       &ShowResults();
   } 

   ## If we drop through to here, it may be a bogus IPv6 address.
   $WhoisResults = " \
        Your query appears to be an IPv6 address but it's not part \
        of a block that I know about.  If you know this is a real \
        IPv6 address, please send mail to $NotifyMail and \
        let us know where queries for it should be directed. ";
   &ShowResults(); 
}

sub CheckSOA
{
   $SOAQuery = $_[0];

   ##
   ## Most of this was lifted from the Michael Fuhr's Net::DNS examples. 
   ##

   $res = new Net::DNS::Resolver;
   $NicServ = "None:  SOA Header Information Checked";
   
   ## Fetch the SOA data if it exists.
   
   $query = $res->query($SOAQuery, "SOA");
   if ($query) {
       $SOAData = ($query->answer)[0]->string;;
   }
   else {
     $WhoisResults = $WhoisResults . "No SOA data appears to exist for this domain.";
       return;
   }
   $WhoisResults = $WhoisResults . "The following SOA data exists for this domain:\n\n";
   $WhoisResults = $WhoisResults . "$SOAData\n\n\n";
   ## Fetch the nameservers if defined.
   my $NSData;
   $res = new Net::DNS::Resolver;
       $query = $res->query($SOAQuery, "NS");
       if ($query) {
           foreach $rr ($query->answer) {
               next unless $rr->type eq "NS";
               $NSData = $NSData . $rr->nsdname . "\n";
           }
       }
       else {
           $NSData = "No nameserver data found.";
       }
   
   $WhoisResults = $WhoisResults . "The following nameservers appear to contain data for this domain:\n\n";
   
   $WhoisResults = $WhoisResults . "$NSData\n";
   
}
